/***************************************************************************
sat_reparaciones.qs  -  description
-------------------
begin                : lun ago 8 2011
copyright            : (C) 2011 by Easy Soft Community
email                : contacto@easysoftcommunity.com
***************************************************************************/

/***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
***************************************************************************/

/** @file */
////////////////////////////////////////////////////////////////////////////
//// DECLARACION ///////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

/** @class_declaration interna */
//////////////////////////////////////////////////////////////////
//// INTERNA /////////////////////////////////////////////////////
class interna {
  var ctx:Object;
  function interna( context ) { this.ctx = context; }
  function init() { this.ctx.interna_init(); }
  function calculateField(fN:String):String { return this.ctx.interna_calculateField(fN); }
  function calculateCounter():String { return this.ctx.interna_calculateCounter(); }
  function validateForm():Boolean { return this.ctx.interna_validateForm(); }
}
//// INTERNA /////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

/** @class_declaration oficial */
//////////////////////////////////////////////////////////////////
//// OFICIAL /////////////////////////////////////////////////////
class oficial extends interna
{
  var refHora:String;
  function oficial( context ) { interna( context ); }
  function bufferChanged(fN:String) {
	return this.ctx.oficial_bufferChanged(fN);
  }
   function grabarDesperfectos() {
 	return this.ctx.oficial_grabarDesperfectos();
   }
  function calcularTotales() {
	return this.ctx.oficial_calcularTotales();
  }
  function generarCostes() {
	return this.ctx.oficial_generarCostes();
  }
  function mostrarTraza() {
	return this.ctx.oficial_mostrarTraza();
  }
}
//// OFICIAL /////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

/** @class_declaration head */
/////////////////////////////////////////////////////////////////
//// DESARROLLO /////////////////////////////////////////////////
class head extends oficial {
  function head( context ) { oficial ( context ); }
}
//// DESARROLLO /////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////

/** @class_declaration ifaceCtx */
/////////////////////////////////////////////////////////////////
//// INTERFACE  /////////////////////////////////////////////////
class ifaceCtx extends head {
  function ifaceCtx( context ) { head( context ); }
}

const iface = new ifaceCtx( this );
//// INTERFACE  /////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////

/** @class_definition interna */
////////////////////////////////////////////////////////////////////////////
//// DEFINICION ////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////
//// INTERNA /////////////////////////////////////////////////////
/** \D Funcion Init
\end */
function interna_init()
{
  var util:FLUtil = new FLUtil();
  var cursor:FLSqlCursor = this.cursor();

  connect(cursor, "bufferChanged(QString)", this, "iface.bufferChanged");
  connect(this.child("tdbLineasReparaciones").cursor(), "bufferCommited()", this, "iface.calcularTotales()");
  connect(this.child("pbnGenerarCostes"), "clicked()", this, "iface.generarCostes");
  connect(this.child("tbnTraza"), "clicked()", this, "iface.mostrarTraza()");

  if (cursor.modeAccess() == cursor.Insert) {
	this.child("fdbCodSerie").setValue(flfactppal.iface.pub_valorDefectoEmpresa("codserie"));
  }
  if (cursor.modeAccess() == cursor.Edit)
	this.child("fdbCodSerie").setDisabled(true);

  this.iface.refHora = "HMN";
}

/** \D Calcula un nuevo nmero de reparacin
\end */
function interna_calculateCounter()
{
  var util:FLUtil = new FLUtil();
  return util.nextCounter("codreparacion", this.cursor());
}

function interna_calculateField(fN:String, cursor:FLSqlCursor):String
{
  var util:FLUtil = new FLUtil();
  var valor:String;
  var cursor:FLSqlCursor = this.cursor();

  switch (fN) {
	/** \C
	El --total-- es el --neto-- menos el --totalirpf-- ms el --totaliva-- ms el --totalrecargo-- ms el --recfinanciero--
	\end */
	case "total":
	  var neto:Number = parseFloat(cursor.valueBuffer("neto"));
	  var totalIva:Number = parseFloat(cursor.valueBuffer("totaliva"));
	  var totalRecargo:Number = parseFloat(cursor.valueBuffer("totalrecargo"));
	  var totalIrpf:Number = parseFloat(cursor.valueBuffer("totalirpf"));
	  valor = neto - totalIrpf + totalIva + totalRecargo;
	  break;
	  /** \C
	  El --neto-- es la suma del pvp total de las lneas de reparacin
	  \end */
	  case "neto":
		valor = util.sqlSelect("sat_lineasreparaciones", "SUM(pvptotal)", "idreparacion = " + cursor.valueBuffer("idreparacion"));
		break;
		/** \C
		El --totaliva-- es la suma del iva correspondiente a las lneas de reparacin
		\end */
		case "totaliva":
		  valor = util.sqlSelect("sat_lineasreparaciones", "SUM((pvptotal * iva) / 100)", "idreparacion = " + cursor.valueBuffer("idreparacion"));
		  valor = parseFloat(util.roundFieldValue(valor, "sat_reparaciones", "totaliva"));
		  break;
		  /** \C
		  El --totarecargo-- es la suma del recargo correspondiente a las lneas de reparacin
		  \end */
		  case "totalrecargo":
			valor = util.sqlSelect("sat_lineasreparaciones", "SUM((pvptotal * recargo) / 100)", "idreparacion = " + cursor.valueBuffer("idreparacion"));
			valor = parseFloat(util.roundFieldValue(valor, "sat_reparaciones", "totalrecargo"));
			break;

		  case "irpf":
			valor = util.sqlSelect("series", "irpf", "codserie = '" + cursor.valueBuffer("codserie") + "'");
			break;
			/** \C
			El --totalirpf-- es el producto del --irpf-- por el --neto--
			\end */
			case "totalirpf":
			  valor = util.sqlSelect("sat_lineasreparaciones", "SUM((pvptotal * irpf) / 100)", "idreparacion = " + cursor.valueBuffer("idreparacion"));
			  // 			valor = (parseFloat(cursor.valueBuffer("irpf")) * (parseFloat(cursor.valueBuffer("neto")))) / 100;
			  valor = parseFloat(util.roundFieldValue(valor, "sat_reparaciones", "totalirpf"));
			  break;

  }
  return valor;
}

function interna_validateForm():Boolean
{
  this.iface.grabarDesperfectos();

  var util:FLUtil = new FLUtil;
  var cursor:FLSqlCursor = this.cursor();

  //Validar Fecha Prevista
  if (cursor.modeAccess() == cursor.Insert) {
	var fechaEnt:String = cursor.valueBuffer("fecha");
	var fechaPrev:String = cursor.valueBuffer("fechaprev");
	debug("FECHAPREV = " + fechaPrev);
	debug("FECHAENT = " + fechaEnt);
	if ( !fechaPrev || fechaPrev < fechaEnt) {
	  MessageBox.warning(util.translate("scripts", "La fecha prevista debe ser igual o posterior a la fecha de entrada."), MessageBox.Ok, MessageBox.NoButton);
	  return false;
	}
  }
}
//// INTERNA /////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////

/** @class_definition oficial */
//////////////////////////////////////////////////////////////////
//// OFICIAL /////////////////////////////////////////////////////

function oficial_calcularTotales()
{
  var util:FLUtil = new FLUtil;

  this.child("fdbNeto").setValue(this.iface.calculateField("neto"));
  this.child("fdbTotalIva").setValue(this.iface.calculateField("totaliva"));
  this.child("fdbTotalRecargo").setValue(this.iface.calculateField("totalrecargo"));
}

function oficial_bufferChanged(fN:String)
{
  var util:FLUtil = new FLUtil();
  var cursor:FLSqlCursor = this.cursor();

  switch (fN) {
	/** \C
	El --total-- es el --neto-- menos el --totalirpf-- ms el --totaliva-- ms el --totalrecargo-- ms el --recfinanciero--
	\end */
	case "neto":
	  this.child("fdbTotaIrpf").setValue(this.iface.calculateField("totalirpf"));
	case "totalrecargo":
	case "totaliva":
	case "totalirpf": {
	  this.child("fdbTotal").setValue(this.iface.calculateField("total"));
	  break;
	}
	/** \C
	El --irpf-- es el asociado a la --codserie-- del albaran
	\end */
	case "codserie": {
	  this.cursor().setValueBuffer("irpf", this.iface.calculateField("irpf"));
	  break;
	}
	/** \C
	El --totalirpf-- es el producto del --irpf-- por el --neto--
	\end */
	case "irpf": {
	  this.child("fdbTotaIrpf").setValue(this.iface.calculateField("totalirpf"));
	  break;
	}
   	/** \C
	Si cambia el cliente borramos el campo codigo de Mquina
	\end */
// 	case "codcliente": {
// 	  this.child("fdbCodMaquinaCli").setValue("");
// 	  break;
// 	}
	/** \C
	Carga de los campos de la mquina a reparar
	\end */
	case "codmaquinacli": {
	  var codMaquinaCli:String = cursor.valueBuffer("codmaquinacli");
	  var codTipoMaq:String = util.sqlSelect("sat_maquinascli", "codtipomaq", "codmaquinacli = '" + codMaquinaCli + "'");
	  var tipoMaq:String = util.sqlSelect("sat_tiposmaq", "tipomaq", "codtipomaq = '" + codTipoMaq + "'");
	  var Modelo:String = util.sqlSelect("sat_maquinascli", "modelo", "codmaquinacli = '" + codMaquinaCli + "'");
	  var satNumSerie:String = util.sqlSelect("sat_maquinascli", "satnumserie", "codmaquinacli = '" + codMaquinaCli + "'");
	  var Color:String = util.sqlSelect("sat_maquinascli", "color", "codmaquinacli = '" + codMaquinaCli + "'");
	  var Desperfectos:String = util.sqlSelect("sat_maquinascli", "desperfectos", "codmaquinacli = '" + codMaquinaCli + "'");

	  this.child("fdbTipoMaq").setValue(tipoMaq);
	  this.child("fdbModelo").setValue(Modelo);
	  this.child("fdbSatNumSerie").setValue(satNumSerie);
	  this.child("fdbColor").setValue(Color);
	  this.child("fdbDesperfectos").setValue(Desperfectos);
	  break;
  	}
  }
}
  /** Al validar el formulario se escribe en la tabla origen sat_maquinascli el campo desperfectos */
 function oficial_grabarDesperfectos()
 {
   debug("oficial_grabarDesperfectos");

   var util:FLUtil = new FLUtil;
   var cursor:FLSqlCursor = this.cursor();
   var curMaquinaCli:FLSqlCursor = new FLSqlCursor("sat_maquinascli");

   var codCliente:String = cursor.valueBuffer("codcliente");
   var CodMaquinaCli:String = cursor.valueBuffer("codmaquinacli");
   var Desperfectos = cursor.valueBuffer("desperfectos");

     curMaquinaCli.select("codmaquinacli = '" + CodMaquinaCli + "'");
     curMaquinaCli.first();
     curMaquinaCli.setModeAccess(curMaquinaCli.Edit);
     curMaquinaCli.refreshBuffer();
     curMaquinaCli.setValueBuffer("desperfectos", Desperfectos);
     if (!curMaquinaCli.commitBuffer()) debug ("Se ha producido un error en grabarDesperfectos()");

 }

function oficial_generarCostes()
{
  var util:FLUtil = new FLUtil();
  var cursor:FLSqlCursor = this.cursor();

  if (cursor.modeAccess() == cursor.Insert) {
	var curDir:FLSqlCursor = this.child("tdbLineasReparaciones").cursor();
	curDir.setModeAccess( curDir.Insert );
	if ( !curDir.commitBufferCursorRelation() )
	  debug( "NO COMMIT" );
}


  // Lnea de horas de mano de obra ////////////////////////////////////////////

  var datosArt = flfactppal.iface.pub_ejecutarQry("articulos", "descripcion,pvp,codimpuesto", "referencia = '" + this.iface.refHora + "'");
  if (datosArt["result"] == -1) {
	MessageBox.warning(util.translate("scripts","No se encuentra el artculo de hora tcnica.\nDebe crearlo con la referencia ") + this.iface.refHora, MessageBox.Ok, MessageBox.NoButton, MessageBox.NoButton);
	return false;
  }

  var codTarifa:String = util.sqlSelect("clientes c INNER JOIN gruposclientes gc ON c.codgrupo = gc.codgrupo", "gc.codtarifa", "codcliente = '" + cursor.valueBuffer("codcliente") + "'", "clientes,gruposclientes");
  if (codTarifa)
	var valor:Number = util.sqlSelect("articulostarifas", "pvp", "referencia = '" + this.iface.refHora + "' AND codtarifa = '" + codTarifa + "'");
  if (valor)
	datosArt["pvp"] = valor;

  // datos de IVA
  var iva:Number = 0;
  var recargo:Number = 0;
  var datosIVA = flfacturac.iface.pub_datosImpuesto(datosArt["codimpuesto"], cursor.valueBuffer("fecha"));
  if (datosIVA)
	iva = datosIVA["iva"];

  var curLineaReparacion:FLSqlCursor = new FLSqlCursor("sat_lineasreparaciones");

  curLineaReparacion.select("idreparacion = " + cursor.valueBuffer("idreparacion") + " AND referencia = '" + this.iface.refHora + "'");
  if (curLineaReparacion.first()) {
	MessageBox.warning(util.translate("scripts","Esta reparacin ya tiene una lnea de mano de obra"), MessageBox.Ok, MessageBox.NoButton, MessageBox.NoButton);
  }
  else {
	var horas:Number = parseInt(cursor.valueBuffer("horas")) + (parseInt(cursor.valueBuffer("minutos")) / 60);

	if (horas == 0) {
	  MessageBox.warning(util.translate("scripts","Para generar la lnea de mano de obra el tiempo empleado debe ser mayor de cero"), MessageBox.Ok, MessageBox.NoButton, MessageBox.NoButton);
	} else {
	  curLineaReparacion.setModeAccess(curLineaReparacion.Insert);
	  curLineaReparacion.refreshBuffer();
	  curLineaReparacion.setValueBuffer("idreparacion", cursor.valueBuffer("idreparacion"));
	  curLineaReparacion.setValueBuffer("referencia", this.iface.refHora);
	  curLineaReparacion.setValueBuffer("descripcion", datosArt["descripcion"]);
	  curLineaReparacion.setValueBuffer("cantidad", horas);
	  curLineaReparacion.setValueBuffer("iva", iva);
	  curLineaReparacion.setValueBuffer("recargo", recargo);
	  curLineaReparacion.setValueBuffer("codimpuesto", datosArt["codimpuesto"]);
	  curLineaReparacion.setValueBuffer("pvpunitario", datosArt["pvp"]);
	  curLineaReparacion.setValueBuffer("pvpsindto", datosArt["pvp"] * horas);
	  curLineaReparacion.setValueBuffer("pvptotal", datosArt["pvp"] * horas);
	  if (!curLineaReparacion.commitBuffer())
		return false;
	}
  }


  this.child("tdbLineasReparaciones").refresh();
  this.iface.calcularTotales();
}

function oficial_mostrarTraza()
{
  flfacturac.iface.pub_mostrarTraza(this.cursor().valueBuffer("codreparacion"), "sat_reparaciones");
}

//// OFICIAL /////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////

/** @class_definition head */
/////////////////////////////////////////////////////////////////
//// DESARROLLO /////////////////////////////////////////////////

//// DESARROLLO /////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
